;===============================================================================
;
; relics boost 1.0
;
; - adjust/adds flags to boost characters status in relics
;
;===============================================================================
header
hirom

;===============================================================================
;
; link
;
;===============================================================================
org     $C20F30

;-------------------------------------------------------------------------------
link:
;-------------------------------------------------------------------------------

    ;link
        JSL boost_main          ;execute boost code

    ;continue
        JMP $0F42

warnpc  $C20F42

;===============================================================================
;
; main code
;
;===============================================================================
org     $C3F700

;-------------------------------------------------------------------------------
boost_main:
;-------------------------------------------------------------------------------

    ;init ----------------------------------------------------------------------
        PHP                     ;backup register
        REP #$10                ;2XY mode
        SEP #$20                ;1AB mode

    ;check vigor boost ---------------------------------------------------------
        LDY #$0B80              ;A = item data offset, B = raise vigor bit
        JSR flag_counter        ;count the number of flags
        BEQ .skip_vigor

    ;setup new vigor
        LDY #$0006              ;vigor ram offset
        JSR status_raise        ;raise vigor
.skip_vigor

    ;check magic boost ---------------------------------------------------------
        LDY #$0504              ;A = item data offset, B = raise magic bit
        JSR flag_counter        ;count the number of flags
        BEQ .skip_magic

    ;setup new magic
        LDY #$0000              ;magic ram offset
        JSR status_raise        ;raise magic
.skip_magic

    ;check stamina boost -------------------------------------------------------
        LDY #$0508              ;A = item data offset, B = raise stamina bit
        JSR flag_counter        ;count the number of flags
        BEQ .skip_stam

    ;setup new magic
        LDY #$0002              ;stamina ram offset
        JSR status_raise        ;raise stamina
.skip_stam

    ;check speed boost ---------------------------------------------------------
        LDY #$0510              ;A = item data offset, B = raise speed bit
        JSR flag_counter        ;count the number of flags
        BEQ .skip_speed

    ;setup new magic
        LDY #$0004              ;speed ram offset
        JSR status_raise        ;raise speed
.skip_speed

    ;exit
        PLP
        RTL
       
;-------------------------------------------------------------------------------
flag_counter:
; Y in =        (item flag, item offset)
; X out =       counter
;-------------------------------------------------------------------------------

    ;init
        REP #$20                ;2AB mode
        TDC                     ;clean 2AB
        PHA                     ;$05,S = counter
        TYA                     ;get item flag and item offset
        AND #$00FF              ;get item flag
        PHA                     ;$03,S = item flag
        TYA                     ;get item flag and item offset
        AND #$FF00              ;get item offset
        XBA                     ;switch A e B
        PHA                     ;$01,S = item offset
        LDY #$0001              ;relic counter

    ;check relic id
.loop_relic
        LDA $11CA,Y             ;item id of relic plus garbage
        AND #$00FF              ;remove garbage
        CMP #$00FF              ;empty item
        BEQ .skip_counter       ;don't count relic flags if empty

    ;calculate offset
        ASL                     ;A = A * 2
        PHA                     ;save in temporary stack
        ASL                     ;A = A * 4
        ASL                     ;A = A * 8
        ASL                     ;A = A * 16
        ASL                     ;A = A * 32
        SEC
        SBC $01,S               ;subtract with stack
        STA $01,S               ;stack = (i * 32) - (i * 2) = (i * 30)
        PLA                     ;get offset and free temporary stack
        CLC
        ADC $01,S               ;add with item offset
        TAX                     ;X = absolute offset

    ;get item flag
        LDA $D85000,X           ;get item flag
        AND $03,S               ;item flag
        BEQ .skip_counter       ;skip if without flag

    ;add counter
        LDA $05,S               ;counter
        INC
        STA $05,S               ;save counter
.skip_counter

    ;get next relic
        DEY                     ;decrement relic slot
        BPL .loop_relic         ;count next relic

    ;exit
        PLA
        PLA
        PLA
        TAX
        SEP #$20                ;mode 1AB
        RTS

;-------------------------------------------------------------------------------
status_raise:
; Y in      = status ram offset
; X in      = counter
;-------------------------------------------------------------------------------

    ;get 0.25 of base value
        LDA $11A0,Y             ;status value
        LSR
        LSR
        PHA                     ;save 0.25 of status value

    ;get value for all boosts 
        TDC                     ;null boost
.boost_loop
        DEX                     ;decrement counter
        BMI .boost_skip
        CLC                     ;add one with adc
        ADC $01,S               ;add 0.25 of status value
        BCS .boost_max          ;cap status value if overflow
        BRA .boost_loop
.boost_skip

    ;get real value
        ADC $11A0,Y             ;add actual boost with base status
        BCC .skip_cap
.boost_max
        LDA #$FF                ;max value
.skip_cap
        STA $11A0,Y             ;update status value

    ;exit
        PLA                     ;free stack
        RTS                     ;exit

warnpc  $C3F7A0
